# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-16 15:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("app", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="blog", name="body_de", field=models.TextField(blank=True, null=True)
        ),
        migrations.AddField(
            model_name="blog", name="body_en", field=models.TextField(blank=True, null=True)
        ),
        migrations.AddField(
            model_name="blog", name="body_fr", field=models.TextField(blank=True, null=True)
        ),
        migrations.AddField(
            model_name="blog", name="body_nl", field=models.TextField(blank=True, null=True)
        ),
        migrations.AddField(
            model_name="blog", name="title_de", field=models.CharField(max_length=255, null=True)
        ),
        migrations.AddField(
            model_name="blog", name="title_en", field=models.CharField(max_length=255, null=True)
        ),
        migrations.AddField(
            model_name="blog", name="title_fr", field=models.CharField(max_length=255, null=True)
        ),
        migrations.AddField(
            model_name="blog", name="title_nl", field=models.CharField(max_length=255, null=True)
        ),
        migrations.AddField(
            model_name="category", name="name_de", field=models.CharField(max_length=255, null=True)
        ),
        migrations.AddField(
            model_name="category", name="name_en", field=models.CharField(max_length=255, null=True)
        ),
        migrations.AddField(
            model_name="category", name="name_fr", field=models.CharField(max_length=255, null=True)
        ),
        migrations.AddField(
            model_name="category", name="name_nl", field=models.CharField(max_length=255, null=True)
        ),
    ]
